//===========================================================================
//
// Editor.h
//
// By Poman So
//
//===========================================================================

#ifndef _Editor_h_
#define _Editor_h_

#include "GUI.h"
#include "3d-tlm.h"

typedef float StkElm[9];
typedef int   Int4[4];
typedef int   Int7[7];
typedef int   Int8[8];


//---------------------------------------------------------------------------
//
//	Editor Class
//
//---------------------------------------------------------------------------
#define ELECTRIC  1
#define MAGNETIC  2
#define GENERAL   4
#define ER_N_UR   8
#define GENERATOR 16
#define SCOPE     32
#define ANIMATOR  64

#define WALL	  7	/* (ELECTRIC|MAGNETIC|GENERAL) */

#define XY_WINDOW 1
#define XZ_WINDOW 2
#define YZ_WINDOW 4

class Editor: public DTaskWindow{
	RadioBox      radiobox1,radiobox2;
	PushButton    databutton;
	Window	      WXYZ;
	DrawingBoard  XY,XZ,YZ;
	Graphic3DBox  XYZ;
	MovableWindow MW;
	Boolean	      Delay;
	int	      radio1,radio2,MW_w,MW_h,StkPtr,StkSize;
	float	xo,yo,zo,dx,dy,dz,Xmin,Ymin,Zmin,Xmax,Ymax,Zmax,rxo,ryo,rzo;
	float	ax0,ay0,az0,ax1,ay1,az1,ax2,ay2,az2,Dx,Dy,Dz,Er_in,Ur_in,
		Refl_in;
	StkElm	*Stack;
	float	Dx1(){return Min(ax1,ax2);}
	float	Dx2(){return Max(ax1,ax2);}
	float	Dy1(){return Min(ay1,ay2);}
	float	Dy2(){return Max(ay1,ay2);}
	float	Dz1(){return Min(az1,az2);}
	float	Dz2(){return Max(az1,az2);}
	FillStyle ReflFill(float refl);
	Color	ReflColor(float refl);
	Boolean	Push(int plane);
	void	PromptData();
	void	GetEquipment(int eq_num, char *prompt, char **eq_list);
	void	Refresh(Boolean clearbackground=True);
	void    RadioBox1_ExposeCB();
	void	DataButton_CB();
	void	WXYZ_ResizeCB();
	void	XY_ActivateCB();
	void	XY_ButtonPressCB();
	void	XY_ButtonReleaseCB();
	void	XY_SelectCB();
	void	XY_ExposeCB();
	void	XY_ScrollCB(int tag);
	void	XZ_ActivateCB();
	void	XZ_ButtonPressCB();
	void	XZ_ButtonReleaseCB();
	void	XZ_ExposeCB();
	void	XZ_SelectCB();
	void	XZ_ScrollCB(int tag);
	void	YZ_ActivateCB();
	void	YZ_ButtonPressCB();
	void	YZ_ButtonReleaseCB();
	void	YZ_SelectCB();
	void	YZ_ExposeCB();
	void	YZ_ScrollCB(int tag);
	void	XYZ_ExposeCB();
	void	MW_ExposeCB();
	void	MW_MoveCB();
public:
	Editor(Window &w, int stkptr=0, int stksize=0, StkElm *stack=0);
	Status	WindowCB(CBTag tag);
	Status	ChildCB(Window *w, CBTag tag);
	fstream	&IO(fstream &fs);
	void	SetDrawParameters();
	void	SetStack(int stkptr, int stksize, StkElm *stack);
	int	StkTop(){return StkPtr;}
	void	GetMeshData(float &cf,float &er,float &ur,int &nx,int &ny,
			    int &nz);
	void	SetMeshData(float cf,float er,float ur,int nx,int ny,int nz);
	void	MeshData(int &nx,int &ny,int &nz,float &x,float &y,float &z);
	int	NumElements(int element_number);
	int	Xnode(float f){return int(f/Dx-0.5);}
	int	Ynode(float f){return int(f/Dy-0.5);}
	int	Znode(float f){return int(f/Dz-0.5);}
	void	GetWallStack(Float7 *stack);
	void	GetErUrStack(Float8 *stack);
	void	GetGeneratorStack(Int8 *stack);
	void	GetScopeStack(Int4 *stack);
	void	GetAnimatorStack(Int7 *stack);
};//--- Editor ---//



#endif 

//===========================================================================
//
// End of Editor.h
//
//===========================================================================
