//===========================================================================
//
// Scope.C
//
// By Poman So
//
//===========================================================================

#ifdef __ActiveCC
#include <string.h>
#endif

#include <Complex>
#include "Scope.h"
#include "3d-tlm.h"
#include "Const.h"
#include <iostream>

using namespace std;
typedef complex<double> Complex;

//---------------------------------------------------------------------------
//
//	Scope Class
//
//---------------------------------------------------------------------------
void Scope::DrawTimeGraph(Boolean allterms){
  if (allterms){
  int i;
    graphbox.Clear();
    graphbox.ToBarGraph();
    // for (i=0;i<NumIts_;i++) graphbox.Line(i,0,i,t[i],Black);
    for (i=1; i<NumIts_; i++)  graphbox.Line(i-1,t[i-1],i,t[i],Black);
  }
  else
    graphbox.Line(NumIts_,0,NumIts_,t[NumIts_],Black);
}//--- Scope::DrawTimeGraph ---//


void Scope::DrawFAmpGraph(){
int    j;
float  fi,fj,df,ai,aj;
double amax,bmax;
  graphbox.Clear();
  graphbox.ToLineGraph();
  for (amax=0,j=0;j<=nf;j++){
    bmax=abs(Complex(Fr[j],Fi[j]));
    if (amax<bmax) amax=bmax;
  }
  if (amax>0.0){
    fi=xmin[1];
    df=(xmax[1]-fi)/nf;
    ai=abs(Complex(Fr[0],Fi[0]))/amax;
    for (fj=fi+df,j=1;j<=nf;j++,fj+=df){
      aj=abs(Complex(Fr[j],Fi[j]))/amax;
      graphbox.Line(fi,ai,fj,aj);
      ai=aj; fi=fj;
    }
  }
  else
    graphbox.Line(xmin[1],0.0,xmax[1],0.0);
}//--- Scope::DrawFAmpGraph ---//


void Scope::DrawFAngGraph(){
int    j;
float  fi,fj,df,ai,aj;
double cf=-180/PI;
  graphbox.Clear();
  graphbox.ToLineGraph();
  fi=xmin[2];
  df=(xmax[2]-fi)/nf;
  ai=(Fr[0]==0)?0:cf*atan2(-Fi[0],Fr[0]);
  for (fj=fi+df,j=1;j<=nf;j++,fj+=df){
    aj=(Fr[j]==0)?0:cf*arg(Complex(Fr[j],Fi[j]));
    graphbox.Line(fi,ai,fj,aj);
    ai=aj; fi=fj;
  }
}//--- Scope::DrawFAngGraph ---//


void Scope::DrawGraph(Boolean allterms){
  if (MeshPtr_ && connected && Mapped()){
  Cursor parentcursor=Parent()->SetCursor(Watch);
    switch (db_statebits){
      case 1: DrawTimeGraph(allterms); break;
      case 2: DrawFAmpGraph();         break;
      case 4: DrawFAngGraph();         break;
    }
    Parent()->SetCursor(parentcursor);
  }
}//--- Scope::DrawGraph ---//


void Scope::GetGraph(){
int i;
float f1,f2;
  switch (db_statebits){
    case 1: i=0; break;
    case 2: i=1; break;
    case 4: i=2; break;
  }
  graphbox.GetTitle(title[i]);
  graphbox.GetHLabel(xlabel[i]);
  graphbox.GetVLabel(ylabel[i]);
  graphbox.GetTicks(xticks[i],yticks[i]);
  if (NumIts_>0 && i>0){
    f1=xmin[1];  f2=xmax[1];
  }
  graphbox.GetWorld(xmin[i],ymin[i],xmax[i],ymax[i]);
  if (NumIts_>0 && i>0){
    xmin[i]=f1;  xmax[i]=f2;
  }
  Parent()->ChildCB(this,DataChangeT);
}//--- Scope::GetGraph ---//


void Scope::SetGraph(){
int i;
  switch (db_statebits){
    case 1: i=0; graphbox.ToBarGraph(); break;
    case 2: i=1; graphbox.ToLineGraph(); break;
    case 4: i=2; graphbox.ToLineGraph(); break;
  }
  graphbox.SetTitle(title[i]);
  graphbox.SetHLabel(xlabel[i]);
  graphbox.SetVLabel(ylabel[i]);
  graphbox.SetTicks(xticks[i],yticks[i]);
  graphbox.SetWorld(xmin[i],ymin[i],xmax[i],ymax[i],True);
}//--- Scope::SetGraph ---//

#define LDFT "Lower DFT frequency bound (GHz)%g"
#define	UDFT "Upper DFT frequency bound (GHz)%g"
#define	NFPS "Number of frequency points%d"

static char *analyzerdata="Scope Data",
	    *prompt2[]={"Stop at (Dt)%d",
			"Iterations per DFT update%d",LDFT,UDFT,NFPS,""
		       };
Status Scope::SelectCB(char *title){
int 	xo=Root()->Width()/4,yo=Root()->Height()/4,
	stopat=StopAt_,step=Step_,NF=nf;
float	F1=xmin[1],F2=xmax[1];
Boolean active[5];
Status	status;
  active[0]=active[1]=True;
  active[2]=active[3]=active[4]=(NumIts_>0)?False:True;
  status = GetActiveData(*Root(),0,xo,yo,xo+350,yo+300,
			 (title)?title:analyzerdata,prompt2,active,
			 &stopat,&step,&F1,&F2,&NF);
  if (status==Okay){
    if (NumIts_>0){F1=xmin[1]; F2=xmax[1]; NF=nf;}
    stopat=Max(Min(stopat,MAX_NUMT),0);
    Step_=Max(step,1);
    if (nf!=NF) nf=Max(Min(MAX_NUMF,NF),1);
    if (NumIts_==0 || stopat!=StopAt_ || xmin[1]!=F1 || xmax[2]!=F2){
    Boolean setgraph=False;
      if (db_statebits==1){
	if (NumIts_==0 || stopat!=StopAt_) setgraph=True;
      }
      else{
	if (xmin[1]!=F1 || xmax[2]!=F2) setgraph=True;
      }
      xmax[0]=StopAt_=stopat;
      xmin[1]=xmin[2]=F1;
      xmax[1]=xmax[2]=F2;
      if (setgraph) SetGraph();
    }
    Parent()->ChildCB(this,DataChangeT);
  }
  return status;
}//--- Scope::SelectCB ---//


void Scope::DFT(){
int	xo=Root()->Width()/4, yo=Root()->Height()/4,NF=nf;
static char *P[]={LDFT,UDFT,NFPS,""};
float	F1=xmin[1],F2=xmax[1];
Scope *PA=Previous();
  if (PA ||
	(GetData(*Root(),0,xo,yo,xo+350,yo+200,"DFT Data",P,&F1,&F2,&NF)==Okay
	&& (F1!=xmin[1] || F2!=xmax[1] || NF!=nf))){
  static Label *WorkInProgress;
  Cursor parentcursor=Parent()->SetCursor(Watch);
  int    n;
    if (PA){
      nf = PA->nf;
      xmin[1]=xmin[2]=PA->xmin[1];
      xmax[1]=xmax[2]=PA->xmax[1];
    }
    else{
      int dx=Parent()->Width()/8,dy=Parent()->Height()/16;
      WorkInProgress = new Label(*Root(),dx*3,dy*7,dx*5,dy*9,1,2,"");
      WorkInProgress->SetLabel("Computing DFT, be patient!");
      Flush();
      Parent()->CheckEvents();
      Flush();
      nf = Max(Min(MAX_NUMF,NF),2);
      xmin[1]=xmin[2]=F1;
      xmax[1]=xmax[2]=F2;
    }
    if (connected){
      for (n=0;n<=nf;n++) Fr[n]=Fi[n]=0;
      for (n=0;n<NumIts_;n++) DFT(n);
      if (db_statebits!=2) domainbox.StateBits(db_statebits=2);
      SetGraph();
    }
    Parent()->SetCursor(parentcursor);
    Parent()->ChildCB(this,DataChangeT);
    if (Next())
      Next()->DFT();
    else
      delete WorkInProgress;
  }
}//--- Scope::DFT ---//

static char *fieldlist[]={"Ex","Ey","Ez","Hx","Hy","Hz"};
static char *domainlist[]={"f(t)","|F|","<F>"},
	    *timedomain="Time Domain Response",
	    *freqdomain="Frequency Domain Response",
	    *timeinDt  ="Time in Delta t",
	    *freqinGHz ="Frequency in GHz",
	    *magnitude ="Magnitude",
	    *amplitude ="Amplitude",
	    *phase     ="Phase";
Scope::Scope(Window &w,char *label,Scope *S):
DTaskWindow(w,100,50,450,350,label,S),
fieldbox(*this,0,titlebar.Y2()+3,50,titlebar.Y2()+120,6,fieldlist,1,True),
domainbox(*this,0,fieldbox.Y2()+2,fieldbox.X2(),fieldbox.Y2()+60,3,domainlist,
	  1,True),
dftbutton(*this,PB_Margin,Height()-PB_Y1fromB,domainbox.X2()-PB_Margin,
	  Height()-PB_Margin,"DFT"),
graphbox(*this,fieldbox.X2()+2,fieldbox.Y1(),Width(),Height()){
  dftbutton.SetRubber(LeftBottomR);
  t  = new float[MAX_NUMT+1];
  Fr = new float[MAX_NUMF+1];
  Fi = new float[MAX_NUMF+1];
  minW=Width();  maxW=MaxWidth();
  minH=Height(); maxH=MaxHeight();
  fb_statebits=fieldbox.StateBits();
  db_statebits=domainbox.StateBits();
  graphbox.SetBorder(0,0);
  graphbox.SetRubber(AllSideR);
  strcpy(title[0],timedomain);
  strcpy(title[1],freqdomain);
  strcpy(title[2],freqdomain);
  strcpy(xlabel[0],timeinDt);
  strcpy(xlabel[1],freqinGHz);
  strcpy(xlabel[2],freqinGHz);
  strcpy(ylabel[0],magnitude);
  strcpy(ylabel[1],amplitude);
  strcpy(ylabel[2],phase);
  xmin[0]=xmin[1]=xmin[2]=0; xmax[0]=StopAt_;  xmax[1]=xmax[2]=10;
  ymin[0]=ymin[1]=0; ymin[2]=-180; ymax[0]=1;    ymax[1]=1;  ymax[2]=180;
  xticks[0]=xticks[1]=xticks[2]=yticks[0]=yticks[1]=yticks[2]=3;
  nf = 100;
  connected = False;
  SetGraph();
}//--- Scope::Scope ---//


Scope::~Scope(){
  delete Fi;
  delete Fr;
  delete t;
}//--- Scope::~Scope ---//


Status Scope::WindowCB(CBTag tag){
  if (tag==ExposeT){
  int x=fieldbox.X2()+1;
    Line(x,0,x,Height(),Black);
  }
  else
    return DTaskWindow::WindowCB(tag);
  return Continue;
}//--- Scope::WindowCB ---//


Status Scope::ChildCB(Window *w, CBTag tag){
  if (w==&deletebutton && ActivatedWindow()==w && tag==ButtonReleaseT){
    Unmap(Watch);
  }
  else if (w==&fieldbox){
    if (tag==ActivateT)
      if (NumIts_==0){
	fb_statebits=fieldbox.StateBits();
	Parent()->ChildCB(this,DataChangeT);
      }
      else
	fieldbox.StateBits(fb_statebits);
  }
  else if (w==&domainbox){
    if (tag==ActivateT){
    int statebits=domainbox.StateBits();
      if (statebits!=db_statebits){
	db_statebits=statebits;
	SetGraph();
      }
    }
  }
  else if (w==&graphbox){
    switch (tag){
      case DataChangeT: GetGraph();  break;
      case ExposeT:     DrawGraph(); break;
      case SelectT:     Parent()->ChildCB(this,UserDef2);  break;
    }
  }
  else if (w==&dftbutton && ActivatedWindow()==w)
    Parent()->ChildCB(this,UserDef3);
  return Continue;
}//--- Scope::ChildCB ---//


fstream &Scope::IO(fstream &fs){
  if (connected){
  int i,DB_STATEBITS,XLEN,YLEN,TLEN;
    fieldbox.IO(fs);
    domainbox.IO(fs);
    fb_statebits=fieldbox.StateBits();
    db_statebits=domainbox.StateBits();
    if (!(fs|xs|ys|zs|nf))
      cerr << "Scope::IO Error!" << endl;
    for (i=0,DB_STATEBITS=1;i<3;i++,DB_STATEBITS<<=1){
      if (db_statebits!=DB_STATEBITS){
	if (iostate_==ios::out){
	  XLEN=strlen(xlabel[i]);
	  YLEN=strlen(ylabel[i]);
	  TLEN=strlen(title[i]);
	}
	if (!(fs|xmin[i]|xmax[i]|xticks[i]|ymin[i]|ymax[i]|yticks[i]|XLEN|YLEN
	        |TLEN))
	  cerr << "Scope::IO Error!" << endl;
	DataIO(fs,xlabel[i],XLEN);
	DataIO(fs,ylabel[i],YLEN);
	DataIO(fs,title[i], TLEN);
	xlabel[i][XLEN]=ylabel[i][YLEN]=title[i][TLEN]=0;
      }
    }
    if (NumIts_>0){
      DataIO(fs,t, sizeof(float)*(NumIts_));
      DataIO(fs,Fr,sizeof(float)*(nf+1));
      DataIO(fs,Fi,sizeof(float)*(nf+1));
    }
    graphbox.IO(fs);
    if (iostate_==ios::in){
      GetGraph();
      if (db_statebits==2){
	xmin[1]=xmin[2];  xmax[1]=xmax[2];
      }
    }
  }
  return (Next())?Next()->IO(fs):fs;
}//--- Scope::IO ---//


void Scope::FreqRes(fstream &fs){
int   i;
float fi=xmin[1],df=(xmax[1]-fi)/nf;
  fs << "Frequency Response" << endl;
  fs << "==================" << endl;
  for(i=0;i<=nf;i++)
    fs << fi+i*df << "  " << Fr[i] << "  " << Fi[i] << endl;
  if (Next()){
    fs << endl << endl;
    Next()->FreqRes(fs);
  }
}//--- Scope::FreqRes ---//


void Scope::TimeRes(fstream &fs){
int   i;
float fi=xmin[1],df=(xmax[1]-fi)/nf;
  fs << "Time Response" << endl;
  fs << "==================" << endl;
  for(i=0;i<NumIts_;i++)
    fs << i << "  " << t[i] << endl;
  if (Next()){
    fs << endl << endl;
    Next()->TimeRes(fs);
  }
}//--- Scope::TimeRes ---//


void Scope::DFT(int n, Boolean forward){
double	wndt,dwndt,In,wt,f1=xmin[1]*1e9,f2=xmax[1]*1e9;
int	i;
  In    = (forward)?t[n]:-t[n];
  dwndt = (wt=TWOPI*n*Dt_)*(f2-f1)/nf;
  wndt  = wt*f1;
  for(i=0;i<=nf;i++,wndt+=dwndt){
    if (wndt>TWOPI) wndt-=TWOPI*int(wndt/TWOPI);
    Fr[i] += In*cos(wndt);
    Fi[i] += In*sin(wndt);
  }
}//--- Scope::DFT ---//


void Scope::Init(){
int i;
  for (i=0;i<=MAX_NUMT;i++) t[i]=0;
  for (i=0;i<=MAX_NUMF;i++) Fr[i]=Fi[i]=0;
  graphbox.Clear();
  if (Next()) Next()->Init();
}//--- Scope::Init ---//


void Scope::Update(Boolean must_refresh,Boolean forward){
VecU ijk(xs,ys,zs);
  if (connected){
    t[NumIts_]=MeshPtr_->GetField(ijk,fb_statebits);
    DFT(NumIts_,forward);
    if (Mapped() && visible_ &&
	(db_statebits==1 || NumIts_%Step_==0 || must_refresh))
      DrawGraph(must_refresh);
  }
  if (Next()) Next()->Update(must_refresh,forward);
}//--- Scope::Update ---//


void Scope::CopySimData(Scope &a){
int i;
  nf = a.nf;
  for (i=0;i<3;i++){xmin[i]=a.xmin[i];  xmax[i]=a.xmax[i];}
  SetGraph();
}//--- Scope::CopySimData ---//


//===========================================================================
//
// End of Scope.C
//
//===========================================================================
