//////////////////////////////////////////////////////////////////////////////
//
// Author:	Poman So
//		=================================================
//		Department of Electrical and Computer Engineering
//		University of Victoria
//		PO Box 3055, MS 8610
//		Victoria, BC, V8X 3P6, Canada
//		-------------------------------------------------
//		Tel:      250-472-4224
//		Fax:      250-721-6052/6230
//		Internet: Poman.So@ECE.UVic.CA
//		=================================================
//
// File:	Stopwatch.h
//
//////////////////////////////////////////////////////////////////////////////
#ifndef _Stopwatch_h_
  #include "Stopwatch.h"
#endif


#ifndef _SYS_TIMES_INCLUDED
  #include <sys/times.h> 
#endif



//////////////////////////////////////////////////////////////////////////////
//
// CLASS: Stopwatch
//
//////////////////////////////////////////////////////////////////////////////


//----------------------------------------------------------------------------
// Stopwatch
//----------------------------------------------------------------------------
Stopwatch::Stopwatch()
  :r_ticks(0),
   s_ticks(0),
   u_ticks(0),
   running(0)
{
}


//----------------------------------------------------------------------------
// Reset
//----------------------------------------------------------------------------
void Stopwatch::Reset()
{
  if (running)
  {
    tms value;
    old_r_ticks = times(&value);
    old_s_ticks = value.tms_stime;
    old_u_ticks = value.tms_utime;
  }
  r_ticks = s_ticks = u_ticks = 0;
}


//----------------------------------------------------------------------------
// Start
//----------------------------------------------------------------------------
void Stopwatch::Start()
{
  if (!running)
  {
    tms value;
    old_r_ticks = times(&value);
    old_s_ticks = value.tms_stime;
    old_u_ticks = value.tms_utime;
    running = 1;
  }
}


//----------------------------------------------------------------------------
// Stop
//----------------------------------------------------------------------------
void Stopwatch::Stop()
{
  if (running)
  {
    tms value;
    r_ticks += times(&value)   - old_r_ticks;
    s_ticks += value.tms_stime - old_s_ticks;
    u_ticks += value.tms_utime - old_u_ticks;
    running = 0;
  }
}




//////////////////////////////////////////////////////////////////////////////
//
//	End of Stopwatch.C
//
//////////////////////////////////////////////////////////////////////////////
