/*==========================================================================*/
/*									    */
/*	XWindow.h							    */
/*									    */
/*	By Poman So							    */
/*==========================================================================*/

#ifndef _XWindow_h_
#define _XWindow_h_


/*
//	--- Display Functions ---
*/

char	 *X_OpenDisplay(void);
void	 X_CloseDisplay(void);
unsigned X_DisplayWidth(void);
unsigned X_DisplayHeight(void);
void	 X_Bell(int percent);
void	 X_FlushDisplay(void);
void	 X_Sync(int discard);

/*
//	--- Window Functions ---
*/
char *X_CreateMainWindow(char *parent, int argc, char **argv,
			 int x, int y, int width, int height,
			 int borderwidth, int cursor, int font);
char *X_CreateWindow(char *parent, int x, int y, int width,
		     int height, int borderwidth, int cursor,
		     int font);
void X_DestroyWindow(char *window);
void X_MapWindow(char *window);
void X_RaiseWindow(char *window);
void X_LowerWindow(char *window);
void X_MapRaiseWindow(char *window);
void X_UnmapWindow(char *window);
void X_GetConfiguration(char *window,int *x,int *y,unsigned *w,unsigned *h);
void X_MoveWindow(char *window, int x, int y);
void X_ResizeWindow(char *window, unsigned w, unsigned h);
void X_MoveResizeWindow(char *window,int x,int y,unsigned w,unsigned h);
void X_SetBorder(char *window, int bw);
void X_SetSubwindowMode(char *window, int wm);
void X_SetCursor(char *window, int cursor);

/*
//	--- Event Handling Functions ---
*/
int  X_NextEvent(int *x, int *y, int *b, int *k, unsigned *t);
int  X_IsEventWindow(char *window);
int  X_MouseAt(char *window, int *x, int *y);
int  X_EventXY(int *x, int *y);
void X_GrabPointer(char *window, int usual);
void X_UngrabPointer(void);
void X_RemoveExposeEvents(void);

/*
//	--- Graphics Functins ---
*/
unsigned X_FontTextWidth(int f, const char *string);
unsigned X_TextWidth(char *window, const char *string);
unsigned X_FontTextHeight(int f, const char *string);
unsigned X_TextHeight(char *window, const char *string);
void	 X_SetFont(char *window, int font);
void	 X_SetFunction(char *window, int f);
void	 X_SetLineAttributes(char *window, int lw, int ls);
void	 X_DrawHText(char *window, int x, int y, const char *string);
void	 X_DrawVText(char *window, int x, int y, const char *string);
void	 X_SetBackground(char *window, int color);
void	 X_SetForeground(char *window, int color);
void	 X_ClearArea(char *window, int x1, int y1, int x2, int y2, int expo);
void	 X_ClearWindow(char *window);
void	 X_Point(char *window, int x, int y);
void	 X_Points(char *window, int n, int *xy);
void	 X_Line(char *window, int x1, int y1, int x2, int y2);
void	 X_Lines(char *window, int n, int *xy);
void	 X_Polygon(char *window, int n, int *xy);
void	 X_FillPolygon(char *window, int n, int *xy);
void	 X_Rectangle(char *window, int x1, int y1, int x2, int y2);
void	 X_Rectangles(char *window, int n, int *xy);
void	 X_FillRectangle(char *window, int x1, int y1, int x2, int y2);
void	 X_FillRectangles(char *window, int n, int *xy);
void	 X_Ellipse(char *window, int x, int y, int xr, int yr);
void	 X_FillEllipse(char *window, int x, int y, int xr, int yr);
void	 X_Surface(char *window, int nx, int ny, int *xy);
void	 X_FillSurface(char *window, int nx, int ny, int *xy, int *depth_code,
	 int fillcolor, int drawcolor);
void	 X_SaveImage(char *window, int x1, int y1, int x2, int y2);
void	 X_RestoreImage(char *window, int x, int y);


void X_ShowColorRuler(char *window);

/*
//	--- Directory Functions ---
*/
int GetFileList(char *indir, char *ext, char ***filelist);
void FreeFileList(char ***filelist);

#endif

/*==========================================================================*/
/*									    */
/*	End of XWindow.h						    */
/*									    */
/*==========================================================================*/
